# DO NOT CHANGE ANY CODE BELOW
# This file is generated automatically by `generate_api_schema.py` and will be overwritten
# Instead, change functions / models by subclassing them in the `./overwrites/` folder. They will be used instead.

import attr

from bungio.models.base import BaseModel


@attr.define
class DestinyStringVariablesComponent(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        integer_values_by_hash: _No description given by bungie._
    """

    integer_values_by_hash: dict[int, int] = attr.field(metadata={"type": """dict[int, int]"""})



from typing import TYPE_CHECKING, Union

import attr

from bungio.models.mixins.user import DestinyUserMixin
from bungio.utils import enum_converter

if TYPE_CHECKING:
    from bungio.models import BungieMembershipType

__all__ = ("DestinyUser",)


@attr.define
class DestinyUser(DestinyUserMixin):
    """
    A representation of a Destiny 2 user

    Attributes:
        membership_id: The user's id
        membership_type: The user's type, aka platform
    """

    membership_id: int = attr.field()
    membership_type: Union["BungieMembershipType", int] = attr.field(converter=enum_converter("BungieMembershipType"))



import attr

from bungio.models.mixins import DestinyClanMixin

__all__ = ("DestinyClan",)


@attr.define
class DestinyClan(DestinyClanMixin):
    """
    A representation of a Destiny 2 clan

    Attributes:
        group_id: The clan's id
    """

    group_id: int = attr.field()



# DO NOT CHANGE ANY CODE BELOW
# This file is generated automatically by `generate_api_schema.py` and will be overwritten
# Instead, change functions / models by subclassing them in the `./overwrites/` folder. They will be used instead.

from typing import TYPE_CHECKING, Optional

import attr

from bungio.models.base import ManifestModel

if TYPE_CHECKING:
    from bungio.models import DestinyDisplayPropertiesDefinition


@attr.define
class DestinyTraitDefinition(ManifestModel):
    """
    _No description given by bungie._

    Tip: Manifest Information
        This model has some attributes which can be filled with additional information found in the manifest (`manifest_...`).
        Without additional work, these attributes will be `None`, since they require additional requests and database lookups.

        To fill the manifest dependent attributes, either:

        - Run `await ThisClass.fetch_manifest_information()`, see [here](/API Reference/Models/base)
        - Set `Client.always_return_manifest_information` to `True`, see [here](/API Reference/client)

    Attributes:
        display_hint: An identifier for how this trait can be displayed. For example: a 'keyword' hint to show an explanation for certain related terms.
        display_properties: _No description given by bungie._
        hash: The unique identifier for this entity. Guaranteed to be unique for the type of entity, but not globally. When entities refer to each other in Destiny content, it is this hash that they are referring to.
        index: The index of the entity as it was found in the investment tables.
        redacted: If this is true, then there is an entity with this identifier/type combination, but BNet is not yet allowed to show it. Sorry!
        trait_category_hash: _No description given by bungie._
        trait_category_id: _No description given by bungie._
        manifest_trait_category_hash: Manifest information for `trait_category_hash`
    """

    display_hint: str = attr.field()
    display_properties: "DestinyDisplayPropertiesDefinition" = attr.field()
    hash: int = attr.field()
    index: int = attr.field()
    redacted: bool = attr.field()
    trait_category_hash: int = attr.field()
    trait_category_id: str = attr.field()
    manifest_trait_category_hash: Optional["DestinyTraitCategoryDefinition"] = attr.field(default=None)


@attr.define
class DestinyTraitCategoryDefinition(ManifestModel):
    """
    _No description given by bungie._

    None
    Attributes:
        hash: The unique identifier for this entity. Guaranteed to be unique for the type of entity, but not globally. When entities refer to each other in Destiny content, it is this hash that they are referring to.
        index: The index of the entity as it was found in the investment tables.
        redacted: If this is true, then there is an entity with this identifier/type combination, but BNet is not yet allowed to show it. Sorry!
        trait_category_id: _No description given by bungie._
        trait_hashes: _No description given by bungie._
        trait_ids: _No description given by bungie._
    """

    hash: int = attr.field()
    index: int = attr.field()
    redacted: bool = attr.field()
    trait_category_id: str = attr.field()
    trait_hashes: list[int] = attr.field(metadata={"type": """list[int]"""})
    trait_ids: list[str] = attr.field(metadata={"type": """list[str]"""})



# DO NOT CHANGE ANY CODE BELOW
# This file is generated automatically by `generate_api_schema.py` and will be overwritten
# Instead, change functions / models by subclassing them in the `./overwrites/` folder. They will be used instead.

import attr

from bungio.models.base import BaseModel


@attr.define
class InterpolationPoint(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        value: _No description given by bungie._
        weight: _No description given by bungie._
    """

    value: int = attr.field()
    weight: int = attr.field()


@attr.define
class InterpolationPointFloat(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        value: _No description given by bungie._
        weight: _No description given by bungie._
    """

    value: float = attr.field()
    weight: float = attr.field()



# DO NOT CHANGE ANY CODE BELOW
# This file is generated automatically by `generate_api_schema.py` and will be overwritten
# Instead, change functions / models by subclassing them in the `./overwrites/` folder. They will be used instead.

from datetime import datetime
from typing import TYPE_CHECKING, Union

import attr

from bungio.models.base import BaseEnum, BaseModel
from bungio.utils import enum_converter

if TYPE_CHECKING:
    from bungio.models import (
        ComponentPrivacySetting,
        ContentItemPublicContract,
        DestinyCharacterActivitiesComponent,
        DestinyCharacterComponent,
        DestinyCharacterProgressionComponent,
        DestinyCharacterRecordsComponent,
        DestinyCharacterRenderComponent,
        DestinyCollectiblesComponent,
        DestinyCraftablesComponent,
        DestinyCurrenciesComponent,
        DestinyEntitySearchResultItem,
        DestinyInventoryComponent,
        DestinyItemComponent,
        DestinyItemInstanceComponent,
        DestinyItemObjectivesComponent,
        DestinyItemPerksComponent,
        DestinyItemPlugComponent,
        DestinyItemPlugObjectivesComponent,
        DestinyItemRenderComponent,
        DestinyItemReusablePlugsComponent,
        DestinyItemSocketsComponent,
        DestinyItemStatsComponent,
        DestinyItemTalentGridComponent,
        DestinyKiosksComponent,
        DestinyMetricsComponent,
        DestinyPlatformSilverComponent,
        DestinyPlugSetsComponent,
        DestinyPresentationNodesComponent,
        DestinyProfileCollectiblesComponent,
        DestinyProfileComponent,
        DestinyProfileProgressionComponent,
        DestinyProfileRecordsComponent,
        DestinyProfileTransitoryComponent,
        DestinyPublicVendorComponent,
        DestinyPublicVendorSaleItemComponent,
        DestinyStringVariablesComponent,
        DestinyVendorCategoriesComponent,
        DestinyVendorComponent,
        DestinyVendorGroupComponent,
        DestinyVendorReceiptsComponent,
        DestinyVendorSaleItemComponent,
        FireteamResponse,
        FireteamSummary,
        GroupBan,
        GroupMember,
        GroupMemberApplication,
        GroupMembership,
        GroupPotentialMembership,
        GroupV2Card,
        PagedQuery,
        PersonalDestinyVendorSaleItemSetComponent,
        PostResponse,
        PublicDestinyVendorSaleItemSetComponent,
        TrendingEntry,
    )


class BungieMembershipType(BaseEnum):
    """
    The types of membership the Accounts system supports. This is the external facing enum used in place of the internal-only Bungie.SharedDefinitions.MembershipType.
    """

    NONE = 0
    """_No description given by bungie._ """
    TIGER_XBOX = 1
    """_No description given by bungie._ """
    TIGER_PSN = 2
    """_No description given by bungie._ """
    TIGER_STEAM = 3
    """_No description given by bungie._ """
    TIGER_BLIZZARD = 4
    """_No description given by bungie._ """
    TIGER_STADIA = 5
    """_No description given by bungie._ """
    TIGER_DEMON = 10
    """_No description given by bungie._ """
    BUNGIE_NEXT = 254
    """_No description given by bungie._ """
    ALL = -1
    """"All" is only valid for searching capabilities: you need to pass the actual matching BungieMembershipType for any query where you pass a known membershipId. """


class BungieCredentialType(BaseEnum):
    """
    The types of credentials the Accounts system supports. This is the external facing enum used in place of the internal-only Bungie.SharedDefinitions.CredentialType.
    """

    NONE = 0
    """_No description given by bungie._ """
    XUID = 1
    """_No description given by bungie._ """
    PSNID = 2
    """_No description given by bungie._ """
    WLID = 3
    """_No description given by bungie._ """
    FAKE = 4
    """_No description given by bungie._ """
    FACEBOOK = 5
    """_No description given by bungie._ """
    GOOGLE = 8
    """_No description given by bungie._ """
    WINDOWS = 9
    """_No description given by bungie._ """
    DEMON_ID = 10
    """_No description given by bungie._ """
    STEAM_ID = 12
    """_No description given by bungie._ """
    BATTLE_NET_ID = 14
    """_No description given by bungie._ """
    STADIA_ID = 16
    """_No description given by bungie._ """
    TWITCH_ID = 18
    """_No description given by bungie._ """


@attr.define
class SearchResultOfContentItemPublicContract(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        has_more: _No description given by bungie._
        query: _No description given by bungie._
        replacement_continuation_token: _No description given by bungie._
        results: _No description given by bungie._
        total_results: _No description given by bungie._
        use_total_results: If useTotalResults is true, then totalResults represents an accurate count. If False, it does not, and may be estimated/only the size of the current page. Either way, you should probably always only trust hasMore. This is a long-held historical throwback to when we used to do paging with known total results. Those queries toasted our database, and we were left to hastily alter our endpoints and create backward- compatible shims, of which useTotalResults is one.
    """

    has_more: bool = attr.field()
    query: "PagedQuery" = attr.field()
    replacement_continuation_token: str = attr.field()
    results: list["ContentItemPublicContract"] = attr.field(metadata={"type": """list[ContentItemPublicContract]"""})
    total_results: int = attr.field()
    use_total_results: bool = attr.field()


@attr.define
class SearchResultOfPostResponse(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        has_more: _No description given by bungie._
        query: _No description given by bungie._
        replacement_continuation_token: _No description given by bungie._
        results: _No description given by bungie._
        total_results: _No description given by bungie._
        use_total_results: If useTotalResults is true, then totalResults represents an accurate count. If False, it does not, and may be estimated/only the size of the current page. Either way, you should probably always only trust hasMore. This is a long-held historical throwback to when we used to do paging with known total results. Those queries toasted our database, and we were left to hastily alter our endpoints and create backward- compatible shims, of which useTotalResults is one.
    """

    has_more: bool = attr.field()
    query: "PagedQuery" = attr.field()
    replacement_continuation_token: str = attr.field()
    results: list["PostResponse"] = attr.field(metadata={"type": """list[PostResponse]"""})
    total_results: int = attr.field()
    use_total_results: bool = attr.field()


@attr.define
class SearchResultOfGroupV2Card(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        has_more: _No description given by bungie._
        query: _No description given by bungie._
        replacement_continuation_token: _No description given by bungie._
        results: _No description given by bungie._
        total_results: _No description given by bungie._
        use_total_results: If useTotalResults is true, then totalResults represents an accurate count. If False, it does not, and may be estimated/only the size of the current page. Either way, you should probably always only trust hasMore. This is a long-held historical throwback to when we used to do paging with known total results. Those queries toasted our database, and we were left to hastily alter our endpoints and create backward- compatible shims, of which useTotalResults is one.
    """

    has_more: bool = attr.field()
    query: "PagedQuery" = attr.field()
    replacement_continuation_token: str = attr.field()
    results: list["GroupV2Card"] = attr.field(metadata={"type": """list[GroupV2Card]"""})
    total_results: int = attr.field()
    use_total_results: bool = attr.field()


@attr.define
class SearchResultOfGroupMember(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        has_more: _No description given by bungie._
        query: _No description given by bungie._
        replacement_continuation_token: _No description given by bungie._
        results: _No description given by bungie._
        total_results: _No description given by bungie._
        use_total_results: If useTotalResults is true, then totalResults represents an accurate count. If False, it does not, and may be estimated/only the size of the current page. Either way, you should probably always only trust hasMore. This is a long-held historical throwback to when we used to do paging with known total results. Those queries toasted our database, and we were left to hastily alter our endpoints and create backward- compatible shims, of which useTotalResults is one.
    """

    has_more: bool = attr.field()
    query: "PagedQuery" = attr.field()
    replacement_continuation_token: str = attr.field()
    results: list["GroupMember"] = attr.field(metadata={"type": """list[GroupMember]"""})
    total_results: int = attr.field()
    use_total_results: bool = attr.field()


@attr.define
class SearchResultOfGroupBan(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        has_more: _No description given by bungie._
        query: _No description given by bungie._
        replacement_continuation_token: _No description given by bungie._
        results: _No description given by bungie._
        total_results: _No description given by bungie._
        use_total_results: If useTotalResults is true, then totalResults represents an accurate count. If False, it does not, and may be estimated/only the size of the current page. Either way, you should probably always only trust hasMore. This is a long-held historical throwback to when we used to do paging with known total results. Those queries toasted our database, and we were left to hastily alter our endpoints and create backward- compatible shims, of which useTotalResults is one.
    """

    has_more: bool = attr.field()
    query: "PagedQuery" = attr.field()
    replacement_continuation_token: str = attr.field()
    results: list["GroupBan"] = attr.field(metadata={"type": """list[GroupBan]"""})
    total_results: int = attr.field()
    use_total_results: bool = attr.field()


@attr.define
class SearchResultOfGroupMemberApplication(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        has_more: _No description given by bungie._
        query: _No description given by bungie._
        replacement_continuation_token: _No description given by bungie._
        results: _No description given by bungie._
        total_results: _No description given by bungie._
        use_total_results: If useTotalResults is true, then totalResults represents an accurate count. If False, it does not, and may be estimated/only the size of the current page. Either way, you should probably always only trust hasMore. This is a long-held historical throwback to when we used to do paging with known total results. Those queries toasted our database, and we were left to hastily alter our endpoints and create backward- compatible shims, of which useTotalResults is one.
    """

    has_more: bool = attr.field()
    query: "PagedQuery" = attr.field()
    replacement_continuation_token: str = attr.field()
    results: list["GroupMemberApplication"] = attr.field(metadata={"type": """list[GroupMemberApplication]"""})
    total_results: int = attr.field()
    use_total_results: bool = attr.field()


@attr.define
class SearchResultOfGroupMembership(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        has_more: _No description given by bungie._
        query: _No description given by bungie._
        replacement_continuation_token: _No description given by bungie._
        results: _No description given by bungie._
        total_results: _No description given by bungie._
        use_total_results: If useTotalResults is true, then totalResults represents an accurate count. If False, it does not, and may be estimated/only the size of the current page. Either way, you should probably always only trust hasMore. This is a long-held historical throwback to when we used to do paging with known total results. Those queries toasted our database, and we were left to hastily alter our endpoints and create backward- compatible shims, of which useTotalResults is one.
    """

    has_more: bool = attr.field()
    query: "PagedQuery" = attr.field()
    replacement_continuation_token: str = attr.field()
    results: list["GroupMembership"] = attr.field(metadata={"type": """list[GroupMembership]"""})
    total_results: int = attr.field()
    use_total_results: bool = attr.field()


@attr.define
class SearchResultOfGroupPotentialMembership(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        has_more: _No description given by bungie._
        query: _No description given by bungie._
        replacement_continuation_token: _No description given by bungie._
        results: _No description given by bungie._
        total_results: _No description given by bungie._
        use_total_results: If useTotalResults is true, then totalResults represents an accurate count. If False, it does not, and may be estimated/only the size of the current page. Either way, you should probably always only trust hasMore. This is a long-held historical throwback to when we used to do paging with known total results. Those queries toasted our database, and we were left to hastily alter our endpoints and create backward- compatible shims, of which useTotalResults is one.
    """

    has_more: bool = attr.field()
    query: "PagedQuery" = attr.field()
    replacement_continuation_token: str = attr.field()
    results: list["GroupPotentialMembership"] = attr.field(metadata={"type": """list[GroupPotentialMembership]"""})
    total_results: int = attr.field()
    use_total_results: bool = attr.field()


@attr.define
class SingleComponentResponseOfDestinyVendorReceiptsComponent(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        data: _No description given by bungie._
        disabled: If true, this component is disabled.
        privacy: _No description given by bungie._
    """

    data: "DestinyVendorReceiptsComponent" = attr.field()
    disabled: bool = attr.field()
    privacy: Union["ComponentPrivacySetting", int] = attr.field(converter=enum_converter("ComponentPrivacySetting"))


@attr.define
class SingleComponentResponseOfDestinyInventoryComponent(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        data: _No description given by bungie._
        disabled: If true, this component is disabled.
        privacy: _No description given by bungie._
    """

    data: "DestinyInventoryComponent" = attr.field()
    disabled: bool = attr.field()
    privacy: Union["ComponentPrivacySetting", int] = attr.field(converter=enum_converter("ComponentPrivacySetting"))


@attr.define
class SingleComponentResponseOfDestinyProfileComponent(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        data: _No description given by bungie._
        disabled: If true, this component is disabled.
        privacy: _No description given by bungie._
    """

    data: "DestinyProfileComponent" = attr.field()
    disabled: bool = attr.field()
    privacy: Union["ComponentPrivacySetting", int] = attr.field(converter=enum_converter("ComponentPrivacySetting"))


@attr.define
class SingleComponentResponseOfDestinyPlatformSilverComponent(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        data: _No description given by bungie._
        disabled: If true, this component is disabled.
        privacy: _No description given by bungie._
    """

    data: "DestinyPlatformSilverComponent" = attr.field()
    disabled: bool = attr.field()
    privacy: Union["ComponentPrivacySetting", int] = attr.field(converter=enum_converter("ComponentPrivacySetting"))


@attr.define
class SingleComponentResponseOfDestinyKiosksComponent(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        data: _No description given by bungie._
        disabled: If true, this component is disabled.
        privacy: _No description given by bungie._
    """

    data: "DestinyKiosksComponent" = attr.field()
    disabled: bool = attr.field()
    privacy: Union["ComponentPrivacySetting", int] = attr.field(converter=enum_converter("ComponentPrivacySetting"))


@attr.define
class SingleComponentResponseOfDestinyPlugSetsComponent(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        data: _No description given by bungie._
        disabled: If true, this component is disabled.
        privacy: _No description given by bungie._
    """

    data: "DestinyPlugSetsComponent" = attr.field()
    disabled: bool = attr.field()
    privacy: Union["ComponentPrivacySetting", int] = attr.field(converter=enum_converter("ComponentPrivacySetting"))


@attr.define
class SingleComponentResponseOfDestinyProfileProgressionComponent(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        data: _No description given by bungie._
        disabled: If true, this component is disabled.
        privacy: _No description given by bungie._
    """

    data: "DestinyProfileProgressionComponent" = attr.field()
    disabled: bool = attr.field()
    privacy: Union["ComponentPrivacySetting", int] = attr.field(converter=enum_converter("ComponentPrivacySetting"))


@attr.define
class SingleComponentResponseOfDestinyPresentationNodesComponent(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        data: _No description given by bungie._
        disabled: If true, this component is disabled.
        privacy: _No description given by bungie._
    """

    data: "DestinyPresentationNodesComponent" = attr.field()
    disabled: bool = attr.field()
    privacy: Union["ComponentPrivacySetting", int] = attr.field(converter=enum_converter("ComponentPrivacySetting"))


@attr.define
class SingleComponentResponseOfDestinyProfileRecordsComponent(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        data: _No description given by bungie._
        disabled: If true, this component is disabled.
        privacy: _No description given by bungie._
    """

    data: "DestinyProfileRecordsComponent" = attr.field()
    disabled: bool = attr.field()
    privacy: Union["ComponentPrivacySetting", int] = attr.field(converter=enum_converter("ComponentPrivacySetting"))


@attr.define
class SingleComponentResponseOfDestinyProfileCollectiblesComponent(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        data: _No description given by bungie._
        disabled: If true, this component is disabled.
        privacy: _No description given by bungie._
    """

    data: "DestinyProfileCollectiblesComponent" = attr.field()
    disabled: bool = attr.field()
    privacy: Union["ComponentPrivacySetting", int] = attr.field(converter=enum_converter("ComponentPrivacySetting"))


@attr.define
class SingleComponentResponseOfDestinyProfileTransitoryComponent(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        data: _No description given by bungie._
        disabled: If true, this component is disabled.
        privacy: _No description given by bungie._
    """

    data: "DestinyProfileTransitoryComponent" = attr.field()
    disabled: bool = attr.field()
    privacy: Union["ComponentPrivacySetting", int] = attr.field(converter=enum_converter("ComponentPrivacySetting"))


@attr.define
class SingleComponentResponseOfDestinyMetricsComponent(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        data: _No description given by bungie._
        disabled: If true, this component is disabled.
        privacy: _No description given by bungie._
    """

    data: "DestinyMetricsComponent" = attr.field()
    disabled: bool = attr.field()
    privacy: Union["ComponentPrivacySetting", int] = attr.field(converter=enum_converter("ComponentPrivacySetting"))


@attr.define
class SingleComponentResponseOfDestinyStringVariablesComponent(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        data: _No description given by bungie._
        disabled: If true, this component is disabled.
        privacy: _No description given by bungie._
    """

    data: "DestinyStringVariablesComponent" = attr.field()
    disabled: bool = attr.field()
    privacy: Union["ComponentPrivacySetting", int] = attr.field(converter=enum_converter("ComponentPrivacySetting"))


@attr.define
class DictionaryComponentResponseOfint64AndDestinyCharacterComponent(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        data: _No description given by bungie._
        disabled: If true, this component is disabled.
        privacy: _No description given by bungie._
    """

    data: dict[int, "DestinyCharacterComponent"] = attr.field(
        metadata={"type": """dict[int, DestinyCharacterComponent]"""}
    )
    disabled: bool = attr.field()
    privacy: Union["ComponentPrivacySetting", int] = attr.field(converter=enum_converter("ComponentPrivacySetting"))


@attr.define
class DictionaryComponentResponseOfint64AndDestinyInventoryComponent(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        data: _No description given by bungie._
        disabled: If true, this component is disabled.
        privacy: _No description given by bungie._
    """

    data: dict[int, "DestinyInventoryComponent"] = attr.field(
        metadata={"type": """dict[int, DestinyInventoryComponent]"""}
    )
    disabled: bool = attr.field()
    privacy: Union["ComponentPrivacySetting", int] = attr.field(converter=enum_converter("ComponentPrivacySetting"))


@attr.define
class DictionaryComponentResponseOfint64AndDestinyCharacterProgressionComponent(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        data: _No description given by bungie._
        disabled: If true, this component is disabled.
        privacy: _No description given by bungie._
    """

    data: dict[int, "DestinyCharacterProgressionComponent"] = attr.field(
        metadata={"type": """dict[int, DestinyCharacterProgressionComponent]"""}
    )
    disabled: bool = attr.field()
    privacy: Union["ComponentPrivacySetting", int] = attr.field(converter=enum_converter("ComponentPrivacySetting"))


@attr.define
class DictionaryComponentResponseOfint64AndDestinyCharacterRenderComponent(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        data: _No description given by bungie._
        disabled: If true, this component is disabled.
        privacy: _No description given by bungie._
    """

    data: dict[int, "DestinyCharacterRenderComponent"] = attr.field(
        metadata={"type": """dict[int, DestinyCharacterRenderComponent]"""}
    )
    disabled: bool = attr.field()
    privacy: Union["ComponentPrivacySetting", int] = attr.field(converter=enum_converter("ComponentPrivacySetting"))


@attr.define
class DictionaryComponentResponseOfint64AndDestinyCharacterActivitiesComponent(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        data: _No description given by bungie._
        disabled: If true, this component is disabled.
        privacy: _No description given by bungie._
    """

    data: dict[int, "DestinyCharacterActivitiesComponent"] = attr.field(
        metadata={"type": """dict[int, DestinyCharacterActivitiesComponent]"""}
    )
    disabled: bool = attr.field()
    privacy: Union["ComponentPrivacySetting", int] = attr.field(converter=enum_converter("ComponentPrivacySetting"))


@attr.define
class DictionaryComponentResponseOfint64AndDestinyKiosksComponent(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        data: _No description given by bungie._
        disabled: If true, this component is disabled.
        privacy: _No description given by bungie._
    """

    data: dict[int, "DestinyKiosksComponent"] = attr.field(metadata={"type": """dict[int, DestinyKiosksComponent]"""})
    disabled: bool = attr.field()
    privacy: Union["ComponentPrivacySetting", int] = attr.field(converter=enum_converter("ComponentPrivacySetting"))


@attr.define
class DictionaryComponentResponseOfint64AndDestinyPlugSetsComponent(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        data: _No description given by bungie._
        disabled: If true, this component is disabled.
        privacy: _No description given by bungie._
    """

    data: dict[int, "DestinyPlugSetsComponent"] = attr.field(
        metadata={"type": """dict[int, DestinyPlugSetsComponent]"""}
    )
    disabled: bool = attr.field()
    privacy: Union["ComponentPrivacySetting", int] = attr.field(converter=enum_converter("ComponentPrivacySetting"))


@attr.define
class DestinyBaseItemComponentSetOfuint32(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        objectives: _No description given by bungie._
        perks: _No description given by bungie._
    """

    objectives: "DictionaryComponentResponseOfuint32AndDestinyItemObjectivesComponent" = attr.field()
    perks: "DictionaryComponentResponseOfuint32AndDestinyItemPerksComponent" = attr.field()


@attr.define
class DictionaryComponentResponseOfuint32AndDestinyItemObjectivesComponent(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        data: _No description given by bungie._
        disabled: If true, this component is disabled.
        privacy: _No description given by bungie._
    """

    data: dict[int, "DestinyItemObjectivesComponent"] = attr.field(
        metadata={"type": """dict[int, DestinyItemObjectivesComponent]"""}
    )
    disabled: bool = attr.field()
    privacy: Union["ComponentPrivacySetting", int] = attr.field(converter=enum_converter("ComponentPrivacySetting"))


@attr.define
class DictionaryComponentResponseOfuint32AndDestinyItemPerksComponent(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        data: _No description given by bungie._
        disabled: If true, this component is disabled.
        privacy: _No description given by bungie._
    """

    data: dict[int, "DestinyItemPerksComponent"] = attr.field(
        metadata={"type": """dict[int, DestinyItemPerksComponent]"""}
    )
    disabled: bool = attr.field()
    privacy: Union["ComponentPrivacySetting", int] = attr.field(converter=enum_converter("ComponentPrivacySetting"))


@attr.define
class DictionaryComponentResponseOfint64AndDestinyPresentationNodesComponent(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        data: _No description given by bungie._
        disabled: If true, this component is disabled.
        privacy: _No description given by bungie._
    """

    data: dict[int, "DestinyPresentationNodesComponent"] = attr.field(
        metadata={"type": """dict[int, DestinyPresentationNodesComponent]"""}
    )
    disabled: bool = attr.field()
    privacy: Union["ComponentPrivacySetting", int] = attr.field(converter=enum_converter("ComponentPrivacySetting"))


@attr.define
class DictionaryComponentResponseOfint64AndDestinyCharacterRecordsComponent(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        data: _No description given by bungie._
        disabled: If true, this component is disabled.
        privacy: _No description given by bungie._
    """

    data: dict[int, "DestinyCharacterRecordsComponent"] = attr.field(
        metadata={"type": """dict[int, DestinyCharacterRecordsComponent]"""}
    )
    disabled: bool = attr.field()
    privacy: Union["ComponentPrivacySetting", int] = attr.field(converter=enum_converter("ComponentPrivacySetting"))


@attr.define
class DictionaryComponentResponseOfint64AndDestinyCollectiblesComponent(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        data: _No description given by bungie._
        disabled: If true, this component is disabled.
        privacy: _No description given by bungie._
    """

    data: dict[int, "DestinyCollectiblesComponent"] = attr.field(
        metadata={"type": """dict[int, DestinyCollectiblesComponent]"""}
    )
    disabled: bool = attr.field()
    privacy: Union["ComponentPrivacySetting", int] = attr.field(converter=enum_converter("ComponentPrivacySetting"))


@attr.define
class DictionaryComponentResponseOfint64AndDestinyStringVariablesComponent(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        data: _No description given by bungie._
        disabled: If true, this component is disabled.
        privacy: _No description given by bungie._
    """

    data: dict[int, "DestinyStringVariablesComponent"] = attr.field(
        metadata={"type": """dict[int, DestinyStringVariablesComponent]"""}
    )
    disabled: bool = attr.field()
    privacy: Union["ComponentPrivacySetting", int] = attr.field(converter=enum_converter("ComponentPrivacySetting"))


@attr.define
class DictionaryComponentResponseOfint64AndDestinyCraftablesComponent(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        data: _No description given by bungie._
        disabled: If true, this component is disabled.
        privacy: _No description given by bungie._
    """

    data: dict[int, "DestinyCraftablesComponent"] = attr.field(
        metadata={"type": """dict[int, DestinyCraftablesComponent]"""}
    )
    disabled: bool = attr.field()
    privacy: Union["ComponentPrivacySetting", int] = attr.field(converter=enum_converter("ComponentPrivacySetting"))


@attr.define
class DestinyBaseItemComponentSetOfint64(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        objectives: _No description given by bungie._
        perks: _No description given by bungie._
    """

    objectives: "DictionaryComponentResponseOfint64AndDestinyItemObjectivesComponent" = attr.field()
    perks: "DictionaryComponentResponseOfint64AndDestinyItemPerksComponent" = attr.field()


@attr.define
class DictionaryComponentResponseOfint64AndDestinyItemObjectivesComponent(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        data: _No description given by bungie._
        disabled: If true, this component is disabled.
        privacy: _No description given by bungie._
    """

    data: dict[int, "DestinyItemObjectivesComponent"] = attr.field(
        metadata={"type": """dict[int, DestinyItemObjectivesComponent]"""}
    )
    disabled: bool = attr.field()
    privacy: Union["ComponentPrivacySetting", int] = attr.field(converter=enum_converter("ComponentPrivacySetting"))


@attr.define
class DictionaryComponentResponseOfint64AndDestinyItemPerksComponent(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        data: _No description given by bungie._
        disabled: If true, this component is disabled.
        privacy: _No description given by bungie._
    """

    data: dict[int, "DestinyItemPerksComponent"] = attr.field(
        metadata={"type": """dict[int, DestinyItemPerksComponent]"""}
    )
    disabled: bool = attr.field()
    privacy: Union["ComponentPrivacySetting", int] = attr.field(converter=enum_converter("ComponentPrivacySetting"))


@attr.define
class DestinyItemComponentSetOfint64(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        instances: _No description given by bungie._
        objectives: _No description given by bungie._
        perks: _No description given by bungie._
        plug_objectives: _No description given by bungie._
        plug_states: _No description given by bungie._
        render_data: _No description given by bungie._
        reusable_plugs: _No description given by bungie._
        sockets: _No description given by bungie._
        stats: _No description given by bungie._
        talent_grids: _No description given by bungie._
    """

    instances: "DictionaryComponentResponseOfint64AndDestinyItemInstanceComponent" = attr.field()
    objectives: "DictionaryComponentResponseOfint64AndDestinyItemObjectivesComponent" = attr.field()
    perks: "DictionaryComponentResponseOfint64AndDestinyItemPerksComponent" = attr.field()
    plug_objectives: "DictionaryComponentResponseOfint64AndDestinyItemPlugObjectivesComponent" = attr.field()
    plug_states: "DictionaryComponentResponseOfuint32AndDestinyItemPlugComponent" = attr.field()
    render_data: "DictionaryComponentResponseOfint64AndDestinyItemRenderComponent" = attr.field()
    reusable_plugs: "DictionaryComponentResponseOfint64AndDestinyItemReusablePlugsComponent" = attr.field()
    sockets: "DictionaryComponentResponseOfint64AndDestinyItemSocketsComponent" = attr.field()
    stats: "DictionaryComponentResponseOfint64AndDestinyItemStatsComponent" = attr.field()
    talent_grids: "DictionaryComponentResponseOfint64AndDestinyItemTalentGridComponent" = attr.field()


@attr.define
class DictionaryComponentResponseOfint64AndDestinyItemInstanceComponent(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        data: _No description given by bungie._
        disabled: If true, this component is disabled.
        privacy: _No description given by bungie._
    """

    data: dict[int, "DestinyItemInstanceComponent"] = attr.field(
        metadata={"type": """dict[int, DestinyItemInstanceComponent]"""}
    )
    disabled: bool = attr.field()
    privacy: Union["ComponentPrivacySetting", int] = attr.field(converter=enum_converter("ComponentPrivacySetting"))


@attr.define
class DictionaryComponentResponseOfint64AndDestinyItemRenderComponent(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        data: _No description given by bungie._
        disabled: If true, this component is disabled.
        privacy: _No description given by bungie._
    """

    data: dict[int, "DestinyItemRenderComponent"] = attr.field(
        metadata={"type": """dict[int, DestinyItemRenderComponent]"""}
    )
    disabled: bool = attr.field()
    privacy: Union["ComponentPrivacySetting", int] = attr.field(converter=enum_converter("ComponentPrivacySetting"))


@attr.define
class DictionaryComponentResponseOfint64AndDestinyItemStatsComponent(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        data: _No description given by bungie._
        disabled: If true, this component is disabled.
        privacy: _No description given by bungie._
    """

    data: dict[int, "DestinyItemStatsComponent"] = attr.field(
        metadata={"type": """dict[int, DestinyItemStatsComponent]"""}
    )
    disabled: bool = attr.field()
    privacy: Union["ComponentPrivacySetting", int] = attr.field(converter=enum_converter("ComponentPrivacySetting"))


@attr.define
class DictionaryComponentResponseOfint64AndDestinyItemSocketsComponent(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        data: _No description given by bungie._
        disabled: If true, this component is disabled.
        privacy: _No description given by bungie._
    """

    data: dict[int, "DestinyItemSocketsComponent"] = attr.field(
        metadata={"type": """dict[int, DestinyItemSocketsComponent]"""}
    )
    disabled: bool = attr.field()
    privacy: Union["ComponentPrivacySetting", int] = attr.field(converter=enum_converter("ComponentPrivacySetting"))


@attr.define
class DictionaryComponentResponseOfint64AndDestinyItemReusablePlugsComponent(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        data: _No description given by bungie._
        disabled: If true, this component is disabled.
        privacy: _No description given by bungie._
    """

    data: dict[int, "DestinyItemReusablePlugsComponent"] = attr.field(
        metadata={"type": """dict[int, DestinyItemReusablePlugsComponent]"""}
    )
    disabled: bool = attr.field()
    privacy: Union["ComponentPrivacySetting", int] = attr.field(converter=enum_converter("ComponentPrivacySetting"))


@attr.define
class DictionaryComponentResponseOfint64AndDestinyItemPlugObjectivesComponent(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        data: _No description given by bungie._
        disabled: If true, this component is disabled.
        privacy: _No description given by bungie._
    """

    data: dict[int, "DestinyItemPlugObjectivesComponent"] = attr.field(
        metadata={"type": """dict[int, DestinyItemPlugObjectivesComponent]"""}
    )
    disabled: bool = attr.field()
    privacy: Union["ComponentPrivacySetting", int] = attr.field(converter=enum_converter("ComponentPrivacySetting"))


@attr.define
class DictionaryComponentResponseOfint64AndDestinyItemTalentGridComponent(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        data: _No description given by bungie._
        disabled: If true, this component is disabled.
        privacy: _No description given by bungie._
    """

    data: dict[int, "DestinyItemTalentGridComponent"] = attr.field(
        metadata={"type": """dict[int, DestinyItemTalentGridComponent]"""}
    )
    disabled: bool = attr.field()
    privacy: Union["ComponentPrivacySetting", int] = attr.field(converter=enum_converter("ComponentPrivacySetting"))


@attr.define
class DictionaryComponentResponseOfuint32AndDestinyItemPlugComponent(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        data: _No description given by bungie._
        disabled: If true, this component is disabled.
        privacy: _No description given by bungie._
    """

    data: dict[int, "DestinyItemPlugComponent"] = attr.field(
        metadata={"type": """dict[int, DestinyItemPlugComponent]"""}
    )
    disabled: bool = attr.field()
    privacy: Union["ComponentPrivacySetting", int] = attr.field(converter=enum_converter("ComponentPrivacySetting"))


@attr.define
class DictionaryComponentResponseOfint64AndDestinyCurrenciesComponent(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        data: _No description given by bungie._
        disabled: If true, this component is disabled.
        privacy: _No description given by bungie._
    """

    data: dict[int, "DestinyCurrenciesComponent"] = attr.field(
        metadata={"type": """dict[int, DestinyCurrenciesComponent]"""}
    )
    disabled: bool = attr.field()
    privacy: Union["ComponentPrivacySetting", int] = attr.field(converter=enum_converter("ComponentPrivacySetting"))


@attr.define
class SingleComponentResponseOfDestinyCharacterComponent(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        data: _No description given by bungie._
        disabled: If true, this component is disabled.
        privacy: _No description given by bungie._
    """

    data: "DestinyCharacterComponent" = attr.field()
    disabled: bool = attr.field()
    privacy: Union["ComponentPrivacySetting", int] = attr.field(converter=enum_converter("ComponentPrivacySetting"))


@attr.define
class SingleComponentResponseOfDestinyCharacterProgressionComponent(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        data: _No description given by bungie._
        disabled: If true, this component is disabled.
        privacy: _No description given by bungie._
    """

    data: "DestinyCharacterProgressionComponent" = attr.field()
    disabled: bool = attr.field()
    privacy: Union["ComponentPrivacySetting", int] = attr.field(converter=enum_converter("ComponentPrivacySetting"))


@attr.define
class SingleComponentResponseOfDestinyCharacterRenderComponent(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        data: _No description given by bungie._
        disabled: If true, this component is disabled.
        privacy: _No description given by bungie._
    """

    data: "DestinyCharacterRenderComponent" = attr.field()
    disabled: bool = attr.field()
    privacy: Union["ComponentPrivacySetting", int] = attr.field(converter=enum_converter("ComponentPrivacySetting"))


@attr.define
class SingleComponentResponseOfDestinyCharacterActivitiesComponent(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        data: _No description given by bungie._
        disabled: If true, this component is disabled.
        privacy: _No description given by bungie._
    """

    data: "DestinyCharacterActivitiesComponent" = attr.field()
    disabled: bool = attr.field()
    privacy: Union["ComponentPrivacySetting", int] = attr.field(converter=enum_converter("ComponentPrivacySetting"))


@attr.define
class SingleComponentResponseOfDestinyCharacterRecordsComponent(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        data: _No description given by bungie._
        disabled: If true, this component is disabled.
        privacy: _No description given by bungie._
    """

    data: "DestinyCharacterRecordsComponent" = attr.field()
    disabled: bool = attr.field()
    privacy: Union["ComponentPrivacySetting", int] = attr.field(converter=enum_converter("ComponentPrivacySetting"))


@attr.define
class SingleComponentResponseOfDestinyCollectiblesComponent(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        data: _No description given by bungie._
        disabled: If true, this component is disabled.
        privacy: _No description given by bungie._
    """

    data: "DestinyCollectiblesComponent" = attr.field()
    disabled: bool = attr.field()
    privacy: Union["ComponentPrivacySetting", int] = attr.field(converter=enum_converter("ComponentPrivacySetting"))


@attr.define
class SingleComponentResponseOfDestinyCurrenciesComponent(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        data: _No description given by bungie._
        disabled: If true, this component is disabled.
        privacy: _No description given by bungie._
    """

    data: "DestinyCurrenciesComponent" = attr.field()
    disabled: bool = attr.field()
    privacy: Union["ComponentPrivacySetting", int] = attr.field(converter=enum_converter("ComponentPrivacySetting"))


@attr.define
class SingleComponentResponseOfDestinyItemComponent(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        data: _No description given by bungie._
        disabled: If true, this component is disabled.
        privacy: _No description given by bungie._
    """

    data: "DestinyItemComponent" = attr.field()
    disabled: bool = attr.field()
    privacy: Union["ComponentPrivacySetting", int] = attr.field(converter=enum_converter("ComponentPrivacySetting"))


@attr.define
class SingleComponentResponseOfDestinyItemInstanceComponent(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        data: _No description given by bungie._
        disabled: If true, this component is disabled.
        privacy: _No description given by bungie._
    """

    data: "DestinyItemInstanceComponent" = attr.field()
    disabled: bool = attr.field()
    privacy: Union["ComponentPrivacySetting", int] = attr.field(converter=enum_converter("ComponentPrivacySetting"))


@attr.define
class SingleComponentResponseOfDestinyItemObjectivesComponent(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        data: _No description given by bungie._
        disabled: If true, this component is disabled.
        privacy: _No description given by bungie._
    """

    data: "DestinyItemObjectivesComponent" = attr.field()
    disabled: bool = attr.field()
    privacy: Union["ComponentPrivacySetting", int] = attr.field(converter=enum_converter("ComponentPrivacySetting"))


@attr.define
class SingleComponentResponseOfDestinyItemPerksComponent(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        data: _No description given by bungie._
        disabled: If true, this component is disabled.
        privacy: _No description given by bungie._
    """

    data: "DestinyItemPerksComponent" = attr.field()
    disabled: bool = attr.field()
    privacy: Union["ComponentPrivacySetting", int] = attr.field(converter=enum_converter("ComponentPrivacySetting"))


@attr.define
class SingleComponentResponseOfDestinyItemRenderComponent(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        data: _No description given by bungie._
        disabled: If true, this component is disabled.
        privacy: _No description given by bungie._
    """

    data: "DestinyItemRenderComponent" = attr.field()
    disabled: bool = attr.field()
    privacy: Union["ComponentPrivacySetting", int] = attr.field(converter=enum_converter("ComponentPrivacySetting"))


@attr.define
class SingleComponentResponseOfDestinyItemStatsComponent(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        data: _No description given by bungie._
        disabled: If true, this component is disabled.
        privacy: _No description given by bungie._
    """

    data: "DestinyItemStatsComponent" = attr.field()
    disabled: bool = attr.field()
    privacy: Union["ComponentPrivacySetting", int] = attr.field(converter=enum_converter("ComponentPrivacySetting"))


@attr.define
class SingleComponentResponseOfDestinyItemTalentGridComponent(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        data: _No description given by bungie._
        disabled: If true, this component is disabled.
        privacy: _No description given by bungie._
    """

    data: "DestinyItemTalentGridComponent" = attr.field()
    disabled: bool = attr.field()
    privacy: Union["ComponentPrivacySetting", int] = attr.field(converter=enum_converter("ComponentPrivacySetting"))


@attr.define
class SingleComponentResponseOfDestinyItemSocketsComponent(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        data: _No description given by bungie._
        disabled: If true, this component is disabled.
        privacy: _No description given by bungie._
    """

    data: "DestinyItemSocketsComponent" = attr.field()
    disabled: bool = attr.field()
    privacy: Union["ComponentPrivacySetting", int] = attr.field(converter=enum_converter("ComponentPrivacySetting"))


@attr.define
class SingleComponentResponseOfDestinyItemReusablePlugsComponent(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        data: _No description given by bungie._
        disabled: If true, this component is disabled.
        privacy: _No description given by bungie._
    """

    data: "DestinyItemReusablePlugsComponent" = attr.field()
    disabled: bool = attr.field()
    privacy: Union["ComponentPrivacySetting", int] = attr.field(converter=enum_converter("ComponentPrivacySetting"))


@attr.define
class SingleComponentResponseOfDestinyItemPlugObjectivesComponent(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        data: _No description given by bungie._
        disabled: If true, this component is disabled.
        privacy: _No description given by bungie._
    """

    data: "DestinyItemPlugObjectivesComponent" = attr.field()
    disabled: bool = attr.field()
    privacy: Union["ComponentPrivacySetting", int] = attr.field(converter=enum_converter("ComponentPrivacySetting"))


@attr.define
class SingleComponentResponseOfDestinyVendorGroupComponent(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        data: _No description given by bungie._
        disabled: If true, this component is disabled.
        privacy: _No description given by bungie._
    """

    data: "DestinyVendorGroupComponent" = attr.field()
    disabled: bool = attr.field()
    privacy: Union["ComponentPrivacySetting", int] = attr.field(converter=enum_converter("ComponentPrivacySetting"))


@attr.define
class DictionaryComponentResponseOfuint32AndDestinyVendorComponent(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        data: _No description given by bungie._
        disabled: If true, this component is disabled.
        privacy: _No description given by bungie._
    """

    data: dict[int, "DestinyVendorComponent"] = attr.field(metadata={"type": """dict[int, DestinyVendorComponent]"""})
    disabled: bool = attr.field()
    privacy: Union["ComponentPrivacySetting", int] = attr.field(converter=enum_converter("ComponentPrivacySetting"))


@attr.define
class DictionaryComponentResponseOfuint32AndDestinyVendorCategoriesComponent(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        data: _No description given by bungie._
        disabled: If true, this component is disabled.
        privacy: _No description given by bungie._
    """

    data: dict[int, "DestinyVendorCategoriesComponent"] = attr.field(
        metadata={"type": """dict[int, DestinyVendorCategoriesComponent]"""}
    )
    disabled: bool = attr.field()
    privacy: Union["ComponentPrivacySetting", int] = attr.field(converter=enum_converter("ComponentPrivacySetting"))


@attr.define
class DestinyVendorSaleItemSetComponentOfDestinyVendorSaleItemComponent(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        sale_items: _No description given by bungie._
    """

    sale_items: dict[int, "DestinyVendorSaleItemComponent"] = attr.field(
        metadata={"type": """dict[int, DestinyVendorSaleItemComponent]"""}
    )


@attr.define
class DictionaryComponentResponseOfuint32AndPersonalDestinyVendorSaleItemSetComponent(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        data: _No description given by bungie._
        disabled: If true, this component is disabled.
        privacy: _No description given by bungie._
    """

    data: dict[int, "PersonalDestinyVendorSaleItemSetComponent"] = attr.field(
        metadata={"type": """dict[int, PersonalDestinyVendorSaleItemSetComponent]"""}
    )
    disabled: bool = attr.field()
    privacy: Union["ComponentPrivacySetting", int] = attr.field(converter=enum_converter("ComponentPrivacySetting"))


@attr.define
class DestinyBaseItemComponentSetOfint32(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        objectives: _No description given by bungie._
        perks: _No description given by bungie._
    """

    objectives: "DictionaryComponentResponseOfint32AndDestinyItemObjectivesComponent" = attr.field()
    perks: "DictionaryComponentResponseOfint32AndDestinyItemPerksComponent" = attr.field()


@attr.define
class DictionaryComponentResponseOfint32AndDestinyItemObjectivesComponent(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        data: _No description given by bungie._
        disabled: If true, this component is disabled.
        privacy: _No description given by bungie._
    """

    data: dict[int, "DestinyItemObjectivesComponent"] = attr.field(
        metadata={"type": """dict[int, DestinyItemObjectivesComponent]"""}
    )
    disabled: bool = attr.field()
    privacy: Union["ComponentPrivacySetting", int] = attr.field(converter=enum_converter("ComponentPrivacySetting"))


@attr.define
class DictionaryComponentResponseOfint32AndDestinyItemPerksComponent(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        data: _No description given by bungie._
        disabled: If true, this component is disabled.
        privacy: _No description given by bungie._
    """

    data: dict[int, "DestinyItemPerksComponent"] = attr.field(
        metadata={"type": """dict[int, DestinyItemPerksComponent]"""}
    )
    disabled: bool = attr.field()
    privacy: Union["ComponentPrivacySetting", int] = attr.field(converter=enum_converter("ComponentPrivacySetting"))


@attr.define
class DestinyItemComponentSetOfint32(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        instances: _No description given by bungie._
        objectives: _No description given by bungie._
        perks: _No description given by bungie._
        plug_objectives: _No description given by bungie._
        plug_states: _No description given by bungie._
        render_data: _No description given by bungie._
        reusable_plugs: _No description given by bungie._
        sockets: _No description given by bungie._
        stats: _No description given by bungie._
        talent_grids: _No description given by bungie._
    """

    instances: "DictionaryComponentResponseOfint32AndDestinyItemInstanceComponent" = attr.field()
    objectives: "DictionaryComponentResponseOfint32AndDestinyItemObjectivesComponent" = attr.field()
    perks: "DictionaryComponentResponseOfint32AndDestinyItemPerksComponent" = attr.field()
    plug_objectives: "DictionaryComponentResponseOfint32AndDestinyItemPlugObjectivesComponent" = attr.field()
    plug_states: "DictionaryComponentResponseOfuint32AndDestinyItemPlugComponent" = attr.field()
    render_data: "DictionaryComponentResponseOfint32AndDestinyItemRenderComponent" = attr.field()
    reusable_plugs: "DictionaryComponentResponseOfint32AndDestinyItemReusablePlugsComponent" = attr.field()
    sockets: "DictionaryComponentResponseOfint32AndDestinyItemSocketsComponent" = attr.field()
    stats: "DictionaryComponentResponseOfint32AndDestinyItemStatsComponent" = attr.field()
    talent_grids: "DictionaryComponentResponseOfint32AndDestinyItemTalentGridComponent" = attr.field()


@attr.define
class DictionaryComponentResponseOfint32AndDestinyItemInstanceComponent(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        data: _No description given by bungie._
        disabled: If true, this component is disabled.
        privacy: _No description given by bungie._
    """

    data: dict[int, "DestinyItemInstanceComponent"] = attr.field(
        metadata={"type": """dict[int, DestinyItemInstanceComponent]"""}
    )
    disabled: bool = attr.field()
    privacy: Union["ComponentPrivacySetting", int] = attr.field(converter=enum_converter("ComponentPrivacySetting"))


@attr.define
class DictionaryComponentResponseOfint32AndDestinyItemRenderComponent(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        data: _No description given by bungie._
        disabled: If true, this component is disabled.
        privacy: _No description given by bungie._
    """

    data: dict[int, "DestinyItemRenderComponent"] = attr.field(
        metadata={"type": """dict[int, DestinyItemRenderComponent]"""}
    )
    disabled: bool = attr.field()
    privacy: Union["ComponentPrivacySetting", int] = attr.field(converter=enum_converter("ComponentPrivacySetting"))


@attr.define
class DictionaryComponentResponseOfint32AndDestinyItemStatsComponent(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        data: _No description given by bungie._
        disabled: If true, this component is disabled.
        privacy: _No description given by bungie._
    """

    data: dict[int, "DestinyItemStatsComponent"] = attr.field(
        metadata={"type": """dict[int, DestinyItemStatsComponent]"""}
    )
    disabled: bool = attr.field()
    privacy: Union["ComponentPrivacySetting", int] = attr.field(converter=enum_converter("ComponentPrivacySetting"))


@attr.define
class DictionaryComponentResponseOfint32AndDestinyItemSocketsComponent(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        data: _No description given by bungie._
        disabled: If true, this component is disabled.
        privacy: _No description given by bungie._
    """

    data: dict[int, "DestinyItemSocketsComponent"] = attr.field(
        metadata={"type": """dict[int, DestinyItemSocketsComponent]"""}
    )
    disabled: bool = attr.field()
    privacy: Union["ComponentPrivacySetting", int] = attr.field(converter=enum_converter("ComponentPrivacySetting"))


@attr.define
class DictionaryComponentResponseOfint32AndDestinyItemReusablePlugsComponent(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        data: _No description given by bungie._
        disabled: If true, this component is disabled.
        privacy: _No description given by bungie._
    """

    data: dict[int, "DestinyItemReusablePlugsComponent"] = attr.field(
        metadata={"type": """dict[int, DestinyItemReusablePlugsComponent]"""}
    )
    disabled: bool = attr.field()
    privacy: Union["ComponentPrivacySetting", int] = attr.field(converter=enum_converter("ComponentPrivacySetting"))


@attr.define
class DictionaryComponentResponseOfint32AndDestinyItemPlugObjectivesComponent(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        data: _No description given by bungie._
        disabled: If true, this component is disabled.
        privacy: _No description given by bungie._
    """

    data: dict[int, "DestinyItemPlugObjectivesComponent"] = attr.field(
        metadata={"type": """dict[int, DestinyItemPlugObjectivesComponent]"""}
    )
    disabled: bool = attr.field()
    privacy: Union["ComponentPrivacySetting", int] = attr.field(converter=enum_converter("ComponentPrivacySetting"))


@attr.define
class DictionaryComponentResponseOfint32AndDestinyItemTalentGridComponent(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        data: _No description given by bungie._
        disabled: If true, this component is disabled.
        privacy: _No description given by bungie._
    """

    data: dict[int, "DestinyItemTalentGridComponent"] = attr.field(
        metadata={"type": """dict[int, DestinyItemTalentGridComponent]"""}
    )
    disabled: bool = attr.field()
    privacy: Union["ComponentPrivacySetting", int] = attr.field(converter=enum_converter("ComponentPrivacySetting"))


@attr.define
class SingleComponentResponseOfDestinyVendorComponent(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        data: _No description given by bungie._
        disabled: If true, this component is disabled.
        privacy: _No description given by bungie._
    """

    data: "DestinyVendorComponent" = attr.field()
    disabled: bool = attr.field()
    privacy: Union["ComponentPrivacySetting", int] = attr.field(converter=enum_converter("ComponentPrivacySetting"))


@attr.define
class SingleComponentResponseOfDestinyVendorCategoriesComponent(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        data: _No description given by bungie._
        disabled: If true, this component is disabled.
        privacy: _No description given by bungie._
    """

    data: "DestinyVendorCategoriesComponent" = attr.field()
    disabled: bool = attr.field()
    privacy: Union["ComponentPrivacySetting", int] = attr.field(converter=enum_converter("ComponentPrivacySetting"))


@attr.define
class DictionaryComponentResponseOfint32AndDestinyVendorSaleItemComponent(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        data: _No description given by bungie._
        disabled: If true, this component is disabled.
        privacy: _No description given by bungie._
    """

    data: dict[int, "DestinyVendorSaleItemComponent"] = attr.field(
        metadata={"type": """dict[int, DestinyVendorSaleItemComponent]"""}
    )
    disabled: bool = attr.field()
    privacy: Union["ComponentPrivacySetting", int] = attr.field(converter=enum_converter("ComponentPrivacySetting"))


@attr.define
class DictionaryComponentResponseOfuint32AndDestinyPublicVendorComponent(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        data: _No description given by bungie._
        disabled: If true, this component is disabled.
        privacy: _No description given by bungie._
    """

    data: dict[int, "DestinyPublicVendorComponent"] = attr.field(
        metadata={"type": """dict[int, DestinyPublicVendorComponent]"""}
    )
    disabled: bool = attr.field()
    privacy: Union["ComponentPrivacySetting", int] = attr.field(converter=enum_converter("ComponentPrivacySetting"))


@attr.define
class DestinyVendorSaleItemSetComponentOfDestinyPublicVendorSaleItemComponent(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        sale_items: _No description given by bungie._
    """

    sale_items: dict[int, "DestinyPublicVendorSaleItemComponent"] = attr.field(
        metadata={"type": """dict[int, DestinyPublicVendorSaleItemComponent]"""}
    )


@attr.define
class DictionaryComponentResponseOfuint32AndPublicDestinyVendorSaleItemSetComponent(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        data: _No description given by bungie._
        disabled: If true, this component is disabled.
        privacy: _No description given by bungie._
    """

    data: dict[int, "PublicDestinyVendorSaleItemSetComponent"] = attr.field(
        metadata={"type": """dict[int, PublicDestinyVendorSaleItemSetComponent]"""}
    )
    disabled: bool = attr.field()
    privacy: Union["ComponentPrivacySetting", int] = attr.field(converter=enum_converter("ComponentPrivacySetting"))


@attr.define
class DestinyItemComponentSetOfuint32(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        instances: _No description given by bungie._
        objectives: _No description given by bungie._
        perks: _No description given by bungie._
        plug_objectives: _No description given by bungie._
        plug_states: _No description given by bungie._
        render_data: _No description given by bungie._
        reusable_plugs: _No description given by bungie._
        sockets: _No description given by bungie._
        stats: _No description given by bungie._
        talent_grids: _No description given by bungie._
    """

    instances: "DictionaryComponentResponseOfuint32AndDestinyItemInstanceComponent" = attr.field()
    objectives: "DictionaryComponentResponseOfuint32AndDestinyItemObjectivesComponent" = attr.field()
    perks: "DictionaryComponentResponseOfuint32AndDestinyItemPerksComponent" = attr.field()
    plug_objectives: "DictionaryComponentResponseOfuint32AndDestinyItemPlugObjectivesComponent" = attr.field()
    plug_states: "DictionaryComponentResponseOfuint32AndDestinyItemPlugComponent" = attr.field()
    render_data: "DictionaryComponentResponseOfuint32AndDestinyItemRenderComponent" = attr.field()
    reusable_plugs: "DictionaryComponentResponseOfuint32AndDestinyItemReusablePlugsComponent" = attr.field()
    sockets: "DictionaryComponentResponseOfuint32AndDestinyItemSocketsComponent" = attr.field()
    stats: "DictionaryComponentResponseOfuint32AndDestinyItemStatsComponent" = attr.field()
    talent_grids: "DictionaryComponentResponseOfuint32AndDestinyItemTalentGridComponent" = attr.field()


@attr.define
class DictionaryComponentResponseOfuint32AndDestinyItemInstanceComponent(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        data: _No description given by bungie._
        disabled: If true, this component is disabled.
        privacy: _No description given by bungie._
    """

    data: dict[int, "DestinyItemInstanceComponent"] = attr.field(
        metadata={"type": """dict[int, DestinyItemInstanceComponent]"""}
    )
    disabled: bool = attr.field()
    privacy: Union["ComponentPrivacySetting", int] = attr.field(converter=enum_converter("ComponentPrivacySetting"))


@attr.define
class DictionaryComponentResponseOfuint32AndDestinyItemRenderComponent(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        data: _No description given by bungie._
        disabled: If true, this component is disabled.
        privacy: _No description given by bungie._
    """

    data: dict[int, "DestinyItemRenderComponent"] = attr.field(
        metadata={"type": """dict[int, DestinyItemRenderComponent]"""}
    )
    disabled: bool = attr.field()
    privacy: Union["ComponentPrivacySetting", int] = attr.field(converter=enum_converter("ComponentPrivacySetting"))


@attr.define
class DictionaryComponentResponseOfuint32AndDestinyItemStatsComponent(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        data: _No description given by bungie._
        disabled: If true, this component is disabled.
        privacy: _No description given by bungie._
    """

    data: dict[int, "DestinyItemStatsComponent"] = attr.field(
        metadata={"type": """dict[int, DestinyItemStatsComponent]"""}
    )
    disabled: bool = attr.field()
    privacy: Union["ComponentPrivacySetting", int] = attr.field(converter=enum_converter("ComponentPrivacySetting"))


@attr.define
class DictionaryComponentResponseOfuint32AndDestinyItemSocketsComponent(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        data: _No description given by bungie._
        disabled: If true, this component is disabled.
        privacy: _No description given by bungie._
    """

    data: dict[int, "DestinyItemSocketsComponent"] = attr.field(
        metadata={"type": """dict[int, DestinyItemSocketsComponent]"""}
    )
    disabled: bool = attr.field()
    privacy: Union["ComponentPrivacySetting", int] = attr.field(converter=enum_converter("ComponentPrivacySetting"))


@attr.define
class DictionaryComponentResponseOfuint32AndDestinyItemReusablePlugsComponent(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        data: _No description given by bungie._
        disabled: If true, this component is disabled.
        privacy: _No description given by bungie._
    """

    data: dict[int, "DestinyItemReusablePlugsComponent"] = attr.field(
        metadata={"type": """dict[int, DestinyItemReusablePlugsComponent]"""}
    )
    disabled: bool = attr.field()
    privacy: Union["ComponentPrivacySetting", int] = attr.field(converter=enum_converter("ComponentPrivacySetting"))


@attr.define
class DictionaryComponentResponseOfuint32AndDestinyItemPlugObjectivesComponent(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        data: _No description given by bungie._
        disabled: If true, this component is disabled.
        privacy: _No description given by bungie._
    """

    data: dict[int, "DestinyItemPlugObjectivesComponent"] = attr.field(
        metadata={"type": """dict[int, DestinyItemPlugObjectivesComponent]"""}
    )
    disabled: bool = attr.field()
    privacy: Union["ComponentPrivacySetting", int] = attr.field(converter=enum_converter("ComponentPrivacySetting"))


@attr.define
class DictionaryComponentResponseOfuint32AndDestinyItemTalentGridComponent(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        data: _No description given by bungie._
        disabled: If true, this component is disabled.
        privacy: _No description given by bungie._
    """

    data: dict[int, "DestinyItemTalentGridComponent"] = attr.field(
        metadata={"type": """dict[int, DestinyItemTalentGridComponent]"""}
    )
    disabled: bool = attr.field()
    privacy: Union["ComponentPrivacySetting", int] = attr.field(converter=enum_converter("ComponentPrivacySetting"))


@attr.define
class SearchResultOfDestinyEntitySearchResultItem(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        has_more: _No description given by bungie._
        query: _No description given by bungie._
        replacement_continuation_token: _No description given by bungie._
        results: _No description given by bungie._
        total_results: _No description given by bungie._
        use_total_results: If useTotalResults is true, then totalResults represents an accurate count. If False, it does not, and may be estimated/only the size of the current page. Either way, you should probably always only trust hasMore. This is a long-held historical throwback to when we used to do paging with known total results. Those queries toasted our database, and we were left to hastily alter our endpoints and create backward- compatible shims, of which useTotalResults is one.
    """

    has_more: bool = attr.field()
    query: "PagedQuery" = attr.field()
    replacement_continuation_token: str = attr.field()
    results: list["DestinyEntitySearchResultItem"] = attr.field(
        metadata={"type": """list[DestinyEntitySearchResultItem]"""}
    )
    total_results: int = attr.field()
    use_total_results: bool = attr.field()


@attr.define
class SearchResultOfTrendingEntry(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        has_more: _No description given by bungie._
        query: _No description given by bungie._
        replacement_continuation_token: _No description given by bungie._
        results: _No description given by bungie._
        total_results: _No description given by bungie._
        use_total_results: If useTotalResults is true, then totalResults represents an accurate count. If False, it does not, and may be estimated/only the size of the current page. Either way, you should probably always only trust hasMore. This is a long-held historical throwback to when we used to do paging with known total results. Those queries toasted our database, and we were left to hastily alter our endpoints and create backward- compatible shims, of which useTotalResults is one.
    """

    has_more: bool = attr.field()
    query: "PagedQuery" = attr.field()
    replacement_continuation_token: str = attr.field()
    results: list["TrendingEntry"] = attr.field(metadata={"type": """list[TrendingEntry]"""})
    total_results: int = attr.field()
    use_total_results: bool = attr.field()


@attr.define
class SearchResultOfFireteamSummary(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        has_more: _No description given by bungie._
        query: _No description given by bungie._
        replacement_continuation_token: _No description given by bungie._
        results: _No description given by bungie._
        total_results: _No description given by bungie._
        use_total_results: If useTotalResults is true, then totalResults represents an accurate count. If False, it does not, and may be estimated/only the size of the current page. Either way, you should probably always only trust hasMore. This is a long-held historical throwback to when we used to do paging with known total results. Those queries toasted our database, and we were left to hastily alter our endpoints and create backward- compatible shims, of which useTotalResults is one.
    """

    has_more: bool = attr.field()
    query: "PagedQuery" = attr.field()
    replacement_continuation_token: str = attr.field()
    results: list["FireteamSummary"] = attr.field(metadata={"type": """list[FireteamSummary]"""})
    total_results: int = attr.field()
    use_total_results: bool = attr.field()


@attr.define
class SearchResultOfFireteamResponse(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        has_more: _No description given by bungie._
        query: _No description given by bungie._
        replacement_continuation_token: _No description given by bungie._
        results: _No description given by bungie._
        total_results: _No description given by bungie._
        use_total_results: If useTotalResults is true, then totalResults represents an accurate count. If False, it does not, and may be estimated/only the size of the current page. Either way, you should probably always only trust hasMore. This is a long-held historical throwback to when we used to do paging with known total results. Those queries toasted our database, and we were left to hastily alter our endpoints and create backward- compatible shims, of which useTotalResults is one.
    """

    has_more: bool = attr.field()
    query: "PagedQuery" = attr.field()
    replacement_continuation_token: str = attr.field()
    results: list["FireteamResponse"] = attr.field(metadata={"type": """list[FireteamResponse]"""})
    total_results: int = attr.field()
    use_total_results: bool = attr.field()


@attr.define
class GlobalAlert(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        alert_html: _No description given by bungie._
        alert_key: _No description given by bungie._
        alert_level: _No description given by bungie._
        alert_link: _No description given by bungie._
        alert_timestamp: _No description given by bungie._
        alert_type: _No description given by bungie._
        stream_info: _No description given by bungie._
    """

    alert_html: str = attr.field()
    alert_key: str = attr.field()
    alert_level: Union["GlobalAlertLevel", int] = attr.field(converter=enum_converter("GlobalAlertLevel"))
    alert_link: str = attr.field()
    alert_timestamp: datetime = attr.field()
    alert_type: Union["GlobalAlertType", int] = attr.field(converter=enum_converter("GlobalAlertType"))
    stream_info: "StreamInfo" = attr.field()


class GlobalAlertLevel(BaseEnum):
    """
    _No description given by bungie._
    """

    UNKNOWN = 0
    """_No description given by bungie._ """
    BLUE = 1
    """_No description given by bungie._ """
    YELLOW = 2
    """_No description given by bungie._ """
    RED = 3
    """_No description given by bungie._ """


class GlobalAlertType(BaseEnum):
    """
    _No description given by bungie._
    """

    GLOBAL_ALERT = 0
    """_No description given by bungie._ """
    STREAMING_ALERT = 1
    """_No description given by bungie._ """


@attr.define
class StreamInfo(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        channel_name: _No description given by bungie._
    """

    channel_name: str = attr.field()



import asyncio
import datetime
import logging
import os
from base64 import b64encode
from copy import copy
from typing import TYPE_CHECKING, Callable, Optional

import attr
from sqlalchemy import MetaData
from sqlalchemy.ext.asyncio import AsyncEngine, create_async_engine

import bungio.singleton as singleton
from bungio.api import ApiClient
from bungio.definitions import DEFAULT_LOGGER, LOGGER_NAME, ROOT_DIR
from bungio.error import InvalidAuthentication
from bungio.http.client import HttpClient
from bungio.manifest import Manifest
from bungio.models import BungieMembershipType
from bungio.models.auth import AuthData
from bungio.models.base import MISSING
from bungio.models.enums import BungieLanguage
from bungio.utils import get_now_with_tz

if TYPE_CHECKING:
    from aiohttp_client_cache import CacheBackend

__all__ = ("Client",)

# define json loading technique
try:
    import orjson

    def json_dumps(x):
        return orjson.dumps(x).decode()

    json_loads = orjson.loads
except ModuleNotFoundError:
    import json

    json_dumps = json.dumps
    json_loads = json.loads


token_update_lock: dict[int, asyncio.Lock] = {}

__all__ = ("Client",)


@attr.define(init=False)
class Client(singleton.Singleton):
    """
    The singleton api client

    Attributes:
        bungie_client_id: The bungie.net client id
        bungie_client_secret: The bungie.net client secret
        bungie_token: The bungie.net token
        logger: If a custom logger should be used
        language: The language the manifest entries should be in
        cache: If http requests should be cached. It is recommended that you make use of this to not ratelimit yourself as quickly.
        manifest_storage: If manifest entires should be stored, and how. Disabling this stops you from querying the manifest.
        always_return_manifest_information: If manifest information should always be returned on every request. Keep in mind that this will increase requests performed / storage for an increased ease of use.
    """

    bungie_client_id: str = attr.field(repr=False)
    bungie_client_secret: str = attr.field(repr=False)
    bungie_token: str = attr.field(repr=False)

    logger: logging.Logger = attr.field(default=DEFAULT_LOGGER)

    language: BungieLanguage = attr.field(default=BungieLanguage.ENGLISH)

    cache: Optional["CacheBackend"] = attr.field(default=None)
    manifest_storage: bool | AsyncEngine = attr.field(default=True)
    always_return_manifest_information: bool = attr.field(default=False)

    json_dumps: Callable = attr.field(init=False, default=json_dumps, repr=False)
    json_loads: Callable = attr.field(init=False, default=json_loads, repr=False)

    api: ApiClient = attr.field(init=False, repr=False)
    http: HttpClient = attr.field(init=False, repr=False)
    manifest: Optional[Manifest] = attr.field(init=False, default=None, repr=False)

    _metadata: Optional[MetaData] = attr.field(init=False, default=None, repr=False)

    _initialised: bool = attr.field(init=False, default=False)

    def __init__(self, *args, **kwargs):
        if not getattr(self, "_initialised", False):
            self.__attrs_init__(*args, **kwargs)
        self._initialised = True

    @manifest_storage.validator  # noqa
    def manifest_storage_check(self, attribute, value):
        if not (isinstance(value, AsyncEngine) or isinstance(value, bool)):
            raise ValueError(f"{attribute} obj must be of type `AsyncEngine`")

    @language.validator  # noqa
    def language_check(self, attribute, value):
        if not isinstance(value, BungieLanguage):
            raise ValueError(f"{attribute} must be an instance of type BungieLanguage")

    def __attrs_post_init__(self):
        if self.always_return_manifest_information:
            if not isinstance(self.manifest_storage, AsyncEngine):
                raise ValueError("Client.manifest_storage must be set up to use this")

        if self.always_return_manifest_information:
            if not isinstance(self.manifest_storage, AsyncEngine):
                raise ValueError("Client.manifest_storage must be set up to use this")

        # save a reference to the client
        singleton.client = self

        # set up the api client
        self.api = ApiClient()

        # set up the http client
        self.http = HttpClient(
            bungie_headers={
                "User-Agent": "BungIO",
                "X-API-Key": self.bungie_token,
                "Accept": "application/json",
            },
            bungie_auth_headers={
                "User-Agent": "BungIO",
                "Content-Type": "application/x-www-form-urlencoded",
                "Authorization": f"""Basic {b64encode(f"{self.bungie_client_id}:{self.bungie_client_secret}".encode()).decode()}""",
                "Accept": "application/json",
            },
        )

        if self.manifest_storage:
            if self.manifest_storage is True:
                # todo clarify where that is stored
                self.manifest_storage = create_async_engine(
                    f"""sqlite+aiosqlite:///{os.path.join(ROOT_DIR, "manifest.db")}"""
                )
            self._metadata = MetaData(bind=self.manifest_storage)
            self.manifest = Manifest()

    def get_auth_url(self, state: str) -> str:
        """
        Generate an url which users can use to authenticate with their bungie.net account.

        Warning: Requires external set up
            This requires you to have set up a redirect url (and logic handling that) on bungie.net.
            More information can be found on [the official documentation](https://github.com/Bungie-net/api/wiki/OAuth-Documentation)

        Args:
            state: An opaque value used by the client to maintain state between the request and the callback. The parameter should be used for preventing cross-site request forgery as described in section 10.12 of the OAuth 2.0 specification.

        Returns:
            The auth url
        """

        return f"https://www.bungie.net/en/oauth/authorize?client_id={self.bungie_client_id}&response_type=code&state={state}"

    async def generate_auth(self, code: str) -> AuthData:
        """
        Generate authentication information from a bungie code. For information on how to get that code, visit the [official documentation](https://github.com/Bungie-net/api/wiki/OAuth-Documentation)

        Tip: Staying up to date
            This dispatches the Client.on_token_update()

        Args:
            code: The code bungie sent

        Raises:
            InvalidAuthentication: If authentication is invalid

        Returns:
            The working authentication info.
        """

        now = get_now_with_tz()
        data = await self.request_access_token(code=code)

        auth = AuthData(
            membership_type=MISSING,
            destiny_membership_id=MISSING,
            bungie_name=MISSING,
            token=data["access_token"],
            token_expiry=now + datetime.timedelta(seconds=data["expires_in"]),
            refresh_token=data["refresh_token"],
            refresh_token_expiry=now + datetime.timedelta(seconds=data["refresh_expires_in"]),
        )
        destiny_info = await self.api.get_membership_data_for_current_user(auth=auth)

        # get the user's destiny info
        # this is not set if the user has no cross save
        auth.membership_id = destiny_info.primary_membership_id or MISSING
        if auth.membership_id is MISSING:
            # if primary is not defined, use the first one and dispatch a info that the user should set up cross save
            auth.membership_id = destiny_info.destiny_memberships[0].membership_id

            # sometimes they don't have cross save set up yet have multiple entries
            if len(destiny_info.destiny_memberships) > 1:
                self.logger.warning(
                    f"User with destiny id `auth.destiny_membership_id` should set up cross save. Had to guess which one to use, since they have multiple accounts -> `{destiny_info.destiny_memberships}`"
                )
                auth.cross_save_setup = False
        auth.membership_id = int(auth.membership_id)

        # get the correct membership type and bungie name
        for entry in destiny_info.destiny_memberships:
            if entry.membership_id == auth.membership_id:
                auth.membership_type = BungieMembershipType(int(entry.membership_type))
                if not entry.bungie_global_display_name or not entry.bungie_global_display_name_code:
                    auth.bungie_name = None
                else:
                    auth.bungie_name = (
                        f"{entry.bungie_global_display_name}#{str(entry.bungie_global_display_name_code).zfill(4)}"
                    )
        assert auth.membership_type is not MISSING

        # dispatch the update event
        asyncio.create_task(self._client.on_token_update(before=None, after=auth))

        return auth

    async def get_working_auth(self, auth: AuthData) -> AuthData:
        """
        Check if tokens need to be refreshed and then do that.
        Gets called automatically when doing requests with AuthData.

        Tip: Staying up to date
            This dispatches the Client.on_token_update()

        Args:
            auth: The potentially old authentication info.

        Raises:
            InvalidAuthentication: If authentication is invalid

        Returns:
            The working authentication info.
        """

        # locked this on a per-user basis
        if auth.membership_id not in token_update_lock:
            token_update_lock.update({auth.membership_id: asyncio.Lock()})
        async with token_update_lock[auth.membership_id]:

            # check that token exists
            if auth.token is None:
                raise InvalidAuthentication(auth)

            # check if token is expired
            now = get_now_with_tz()
            if auth.token_expiry < (now - datetime.timedelta(minutes=5)):
                return auth

            # check the refresh token expiry
            if auth.refresh_token_expiry > (now - datetime.timedelta(minutes=5)):
                self._invalidate_token(auth=auth)
                raise InvalidAuthentication(auth)

            old_auth = copy(auth)

            # refresh the data
            data = await self.refresh_access_token(auth=auth)
            auth.token = data["access_token"]
            auth.refresh_token = data["refresh_token"]
            auth.token_expiry = now + datetime.timedelta(seconds=data["expires_in"])
            auth.refresh_expires_in = now + datetime.timedelta(seconds=data["refresh_expires_in"])

            # dispatch the update event
            asyncio.create_task(self._client.on_token_update(before=old_auth, after=auth))

        return auth

    async def on_token_update(self, before: Optional[AuthData], after: AuthData) -> None:
        """
        Dispatched whenever a token is generated, updated, or invalidated.

        Tip: Subclassing
            It is highly recommended to subclass the Client and overwrite this function to suite your own needs

        Note: Generation
            If the first token is generated, `before` will be `None`.

        Note: Invalidation
            The token can be invalidated if the auth data is expired. To prevent that, make sure your tokens are updated regularly.
            If the token is invalidated, `after.token` will be `None`.

        Args:
            before: The old auth info
            after: The new auth info
        """

        self.logger.info(f"Updated token for {before.membership_id=}: {before.token=} -> {after.token=}")



# DO NOT CHANGE ANY CODE BELOW
# This file is generated automatically by `generate_api_schema.py` and will be overwritten
# Instead, change functions / models by subclassing them in the `./overwrites/` folder. They will be used instead.

from typing import Optional

import attr

from bungio.models import CoreSystem
from bungio.models.auth import AuthData
from bungio.models.base import ClientMixin
from bungio.utils import AllowAsyncIteration


@attr.define
class UserSystemOverridesRouteInterface(ClientMixin):
    async def get_user_system_overrides(self, auth: Optional[AuthData] = None) -> dict[str, CoreSystem]:
        """
        Get the user-specific system overrides that should be respected alongside common systems.

        Args:
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        response = await self._client.http.get_user_system_overrides(auth=auth)
        return {
            key: await CoreSystem.from_dict(data=value, client=self._client, auth=auth)
            async for key, value in AllowAsyncIteration(response["Response"].items())
        }
